@@ -6,6 +6,8 @@ from datetime import datetime
 from OpenSSL import crypto
 import M2Crypto as m2crypto
 
+from django.contrib.sites.models import Site
+
 #XXX GOD KILLS A KITTEN EVERY TIME
 #WE USE BOTH WRAPPERS TOGETHER!!!
 #Hopefully, Santa bring us an improved m2crypto API
@@ -74,7 +76,7 @@ class CertCreator(object):
                 "expires_ts",
                 )
         for k in POSSIBLE_OVERRIDES:
-            if kwargs.has_key(k):
+            if k in kwargs:
                 overrides[k] = kwargs[k]
 
         #validity overrides
@@ -92,13 +94,12 @@ class CertCreator(object):
         #might split...
 
         #issuerKey override (for test purposes mainly)
-        issuerKey = kwargs.get('issuerKey',None)
+        issuerKey = kwargs.get('issuerKey', None)
         if issuerKey:
             self.issuerKey = issuerKey
 
         self.opts = opt
 
-
     def _get_user_data(self, user):
         from django_webid.provider.models import WebIDUser
         u = WebIDUser.get_for_user(user)
@@ -129,8 +130,8 @@ class CertCreator(object):
         if "BEGINIECERTIFICATEREQUEST" in spkac_str:
             #XXX improve csr type detection (get a flag from view)
             pkcs10 = spkac_str.replace(
-                    '-----BEGINIECERTIFICATEREQUEST-----','').replace(
-                    '-----ENDIECERTIFICATEREQUEST-----','')
+                    '-----BEGINIECERTIFICATEREQUEST-----', '').replace(
+                    '-----ENDIECERTIFICATEREQUEST-----', '')
 
             der = base64.b64decode(pkcs10)
             self.csr = crypto.load_certificate_request(
@@ -153,10 +154,8 @@ class CertCreator(object):
         """
         self.cert = crypto.X509()
 
-
     # ***
     # begins crypto functions that alter the certificate
-
     def _create_key_pair(self, _type, bits):
         pkey = crypto.PKey()
         pkey.generate_key(_type, bits)
@@ -174,7 +173,7 @@ class CertCreator(object):
         issuer_data = (('CN', 'Not a Certification Authority'),
                         ('O', 'FOAF+SSL'),
                         ('OU', 'The Community of Self Signers'))
-        for k,v in issuer_data:
+        for k, v in issuer_data:
             setattr(icert.get_subject(), k, v)
         self.icert = icert
 
@@ -187,8 +186,14 @@ class CertCreator(object):
                 self.opts.cn_field,
                 None)
         if not cn_field:
-            cn_field = gettattr(self.webiduser,
+            cn_field = getattr(self.webiduser,
                     'username')
+        try:
+            domain = Site.objects.get_current().domain
+        except:
+            domain = None
+        if domain:
+            cn_field = "%s@%s" % (cn_field, domain)
         self.cert.get_subject().CN = cn_field
 
     def _set_pubkey(self):
@@ -274,17 +279,16 @@ class CertCreator(object):
         mod = pubkey.get_modulus().lower()
 
         pkrsa = pubkey.get_rsa()
-        e  = m2crypto.m2.rsa_get_e(pkrsa.rsa)
+        e = m2crypto.m2.rsa_get_e(pkrsa.rsa)
         ha = binascii.hexlify(e)
         #not correct
         #but it's not likely >3
         nbytes = ha.lstrip('0')[0]
-        exp = int(ha[-int(nbytes)*2:],16)
+        exp = int(ha[-int(nbytes) * 2:], 16)
         bits = pubkey.size() * 8
 
         return (mod, exp, bits)
 
-
     ##############################
     # done with crypto functions
     # Now: writing model instances
@@ -293,7 +297,6 @@ class CertCreator(object):
     #XXX can we decouple the instance writting
     #from the crypto functions??
     #--- As two separate classes...
-
     def _set_cert_serial_number(self, serial):
         self.cert.set_serial_number(serial)
 
@@ -329,7 +332,6 @@ class CertCreator(object):
             filetype,
             self.cert)
 
-
     def _get_cert_hashes(self):
         """
         returns sha1 and md5 hashes of the
@@ -337,7 +339,7 @@ class CertCreator(object):
         strings.
         """
         import hashlib
-        beautify = lambda s: ':'.join([s[i:i+2] \
+        beautify = lambda s: ':'.join([s[i:i + 2] \
                 for i in xrange(0, len(s), 2)])
         return (beautify(
                     hashlib.sha1(self.cert_dump).\
@@ -367,7 +369,6 @@ class CertCreator(object):
         c.user_agent_string = self.user_agent_string
         c.save()
 
-
     #####################################################
     #####################################################
     # Public Methods
@@ -431,7 +432,6 @@ class CertCreator(object):
         return self.fingerprint_sha1
 
 
-
 ###############################################
 ###############################################
 # SOME OTHER OLD FUNCTIONS FROM our legacy code
@@ -439,7 +439,6 @@ class CertCreator(object):
 ###############################################
 # To be cleaned and reused
 # (pkcs12, i'm looking at you)
-
 def get_serial_from_file(serial_path='/tmp/webid_cert_serial.txt'):
     """Get serial number from file
 
@@ -456,7 +455,8 @@ def get_serial_from_file(serial_path='/tmp/webid_cert_serial.txt'):
         serial_file = open(serial_path, "r")
         data = serial_file.read()
         serial_file.close()
-        if data: number = int(data)
+        if data:
+            number = int(data)
         number += 1
     except:
         number = 1
@@ -465,6 +465,7 @@ def get_serial_from_file(serial_path='/tmp/webid_cert_serial.txt'):
     serial_file.close()
     return number
 
+
 def gen_keypair(bits=1024):
     """Create RSA key pair
     Equivalent to: openssl genrsa -des3 -out client.key 1024
@@ -473,7 +474,6 @@ def gen_keypair(bits=1024):
     :type bits: int
     :return: key
     :rtype: EVP.PKey
-
     """
     pkey = EVP.PKey()
     rsa = RSA.gen_key(bits, 65537)
@@ -492,9 +492,7 @@ def gen_csr(pkey):
     :type pkey: EVP.PKey
     :return: x509 request
     :rtype: X509.Request
-
     """
-
     csr = X509.Request()
     csr.set_pubkey(pkey)
     return csr
@@ -636,8 +634,6 @@ def gen_httpwebid_selfsigned_cert_pemfile(webid,
     save_pkey_cert_to_pemfile(cert, pkey, cert_path, key_path)
     return cert_path, key_path
 
-
-
 ##########################################
 # PKCS12 SHIT
 # ANOTHER SPOT WHERE WE NEED PYOPENSSL !!!
